import { RegExpMapped, SyntaxDefinition } from './syntax';
export type { SyntaxDefinition, RegExpMapped };
declare global {
    interface CSSStyleSheet {
        replaceSync(css: string): void;
    }
    interface ShadowRoot {
        adoptedStyleSheets: readonly CSSStyleSheet[];
    }
}
export interface HTMLCodeSyntaxElement {
    language?: string;
    theme?: string;
    syntax?: SyntaxDefinition | Promise<{
        default: SyntaxDefinition;
    }>;
    html?: string;
    styleSheet?: CSSStyleSheet;
}
export declare const createStyleSheet: (keys: string[]) => CSSStyleSheet;
export declare const languages: Record<string, SyntaxDefinition | Promise<{
    default: SyntaxDefinition;
}>>;
/**
 * CodeSyntax custom element
 *
 * ```js
 * import { CodeSyntaxElement } from 'code-syntax'
 * customElements.define('code-syntax', CodeSyntaxElement)
 * ```
 */
export declare class CodeSyntaxElement extends HTMLElement {
    #private;
    static get observedAttributes(): string[];
    pins: {
        code: HTMLElement;
    };
    constructor();
    highlight: (s: string) => string;
    render: () => void;
    set html(html: string);
    set syntax(def: SyntaxDefinition | Promise<{
        default: SyntaxDefinition;
    }>);
    set styleSheet(styleSheet: CSSStyleSheet);
    setSyntaxDefinition(def: SyntaxDefinition | Promise<{
        default: SyntaxDefinition;
    }>): Promise<void>;
    attributeChangedCallback(name: string, _: string | null, newValue: string | null): void;
}
